/*

	ADC (Analog to Digital Converter) related things.
	
	The ADC measures voltage at 4 I/O ports consecutively and stores
	the measurement results into array 'adc'.
	
*/

#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>

volatile uint16_t adc[4];
volatile uint8_t next;

void adc_switch(uint8_t channel){
/*
	Switches ADC channel to
		
		ADC3	channel = 0		Acceleration seonsor ADXL335 X
		ADC4	channel = 1		Acceleration seonsor ADXL335 Y
		ADC5	channel = 2		Acceleration seonsor ADXL335 Z
		ADC1	channel = 3		Photodiode (Transimpedance amplifier)
*/
	
	switch(channel){
		
		case 0:
		
			ADMUX &= ~(1 << MUX2);
			ADMUX |= (1 << MUX0) | (1 << MUX1);
			break;
			
		case 1:
		
			ADMUX &= ~((1 << MUX0) | (1 << MUX1));
			ADMUX |= (1 << MUX2);
			break;
			
		case 2:
		
			ADMUX &= ~(1 << MUX1);
			ADMUX |= (1 << MUX0) | (1 << MUX2);
			break;
		
		default:
		
			ADMUX &= ~((1 << MUX1) | (1 << MUX2));
			ADMUX |= (1 << MUX0);
	}
}

void adc_init(){

	/* AVCC as reference voltage */
	/* Enable (ADC) interrupts */
	/* Prescaler 8 */

	ADMUX |= (1 << REFS0);
	ADCSRA |= (1 << ADIE);
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1);
	ADCSRA |= (1 << ADEN);
	
	next = 0;
	adc_switch(next);
	
	/* Start measurement */
	ADCSRA |= (1 << ADSC);
}

ISR(ADC_vect){
	
	
	adc[next] = ADCL | (ADCH << 8);

	
	++next;
	if(next == 4){
		
		next = 0;
	}
	
	adc_switch(next);
	ADCSRA |= (1 << ADSC);
}


