/*
	
	Things related with the infrared key device.
	
*/

#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>


volatile uint16_t code[2][32];
volatile uint8_t pointer = 0;
volatile uint8_t ready = 0;

extern volatile uint16_t adc[4];

void key_init(){
	
	/* Prescaler 256 */
	/* Enable overvlow interrupts */
	
	TCCR0B |= (1 << CS02);
	TIMSK0 |= (1 << TOIE0);
}

uint8_t key_submitted(uint32_t key){
/*
	Checks if received code (during last measurement cycle)
	matches given key.
*/

	uint32_t avrg;
	uint32_t tmp;
	uint8_t i, j;
	
	if(ready){
		
		avrg = 0;
	
		for(i = 0; i < 32 * 2; ++i){
		
			avrg += code[i % 2][i >> 1];
		}
	
		avrg /= 64;
	
		for(i = 0; i < 32 * 2; ++i){
		
			tmp = 0;
		
			for(j = 0; j < 32; ++j){
			
				if(code[i % 2][(i / 2 + j) % 32] > avrg){
				
					tmp |= (((uint32_t) 1) << j);
				}
			}
		
			if(tmp == key){
				
				ready = 0;
				return 1;
			}
		}
			
		ready = 0;
	}
	
	return 0;
}


ISR(TIMER0_OVF_vect){
		
	if(!ready){
	
		code[pointer % 2][pointer >> 1] = adc[3];
		++pointer;
	}
	
	if(pointer == 32 * 2){
		
		pointer = 0;
		ready = 1;
	}
}
